/**
 * @license
 * Copyright Akveo. All Rights Reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 */
import { Component, OnInit } from '@angular/core';
import { NavigationEnd, NavigationStart, Router } from '@angular/router';
import { NbMenuItem } from '@nebular/theme';
import { PageMenuService } from './core/services/page-menu.service';
import { BaseComponent } from './shared/components/base/base.component';
import { combineLatest } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { AuthService } from './core/services/auth.service';
import { AppStateService } from './core/services/app.state.service';

@Component({
  selector: 'ngx-app',
  styleUrls: ['./app.component.scss'],
  template: `
  <div id="root">
    <div class="content">
      <ngx-one-column-layout [showMenu]="menu.length != 0" *ngIf="init">
        <nb-menu [items]="menu"></nb-menu>
        <router-outlet appRouteReuseLifecycle></router-outlet>
      </ngx-one-column-layout>
    </div>
    <div class="overlay">
    </div>
  </div>
  `,
})
export class AppComponent extends BaseComponent {
  menu: NbMenuItem[] = [];

  init = false;

  constructor(
    private authService: AuthService,
    private appStateService: AppStateService,
    pageMenu: PageMenuService,
    private router: Router) {
    super();

    
    pageMenu.menu$
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(menu => {
        this.menu = menu;
      });

    router.events
      .pipe(takeUntil(this.unsubscribe))
      .subscribe((val) => {
        if (val instanceof NavigationStart) {
          if (!this.init) {
            this.initApp(val.url);
          }
        }
      });

    appStateService.init$
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(val => {
        this.init = val;
      });
  }

  initApp(url: string): void {
    combineLatest(this.authService.loadPrincipal(), this.appStateService.loadTerm())
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(([principal, term]) => {
        this.appStateService.setInit(true);
        if (principal != null && this.appStateService.currentUrl == '/') {
          this.router.navigate(["/dashboard"]);
        }
      });
  }
}
