import { Component, Input, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { TaskInstance } from '../../models/task-instance';
import { ExerciseProgress } from '../../models/exercise-progress';

@Component({
  selector: 'ngx-exercise-progress',
  templateUrl: './exercise-progress.component.html',
  styleUrls: ['./exercise-progress.component.scss']
})
export class ExerciseProgressComponent implements OnInit {
  @Input() _exerciseProgresses: ExerciseProgress[] = [];
  filteredExerciseProgresses: ExerciseProgress[] = [];
  @Input() showFilter: boolean;
  hideInvisible = true;

  @Input() set exerciseProgresses(val: ExerciseProgress[]) {
    this._exerciseProgresses = val;
    console.log(this.hideInvisible);
    this.applyFilter(this.hideInvisible);
  }

  get exerciseProgresses(): ExerciseProgress[] {
    return this._exerciseProgresses;
  }

  constructor(private router: Router) { }

  ngOnInit(): void {
  }

  navigateToSubmission(exerciseProgress: ExerciseProgress) {
    if(!exerciseProgress.submission) return;
    this.router.navigate(['/', 'submissions', exerciseProgress.submission.id]);
  }

  hasSolution(exerciseResult: ExerciseProgress, taskInstance: TaskInstance) {
    return this.getSolution(exerciseResult, taskInstance) != null;
  }

  getSolution(exerciseResult: ExerciseProgress, taskInstance: TaskInstance) {
    return exerciseResult.solutions.find(s => s.task.id == taskInstance.task.id);
  }

  hasPointRating(exerciseResult: ExerciseProgress, taskInstance: TaskInstance) {
    return this.getPointRating(exerciseResult, taskInstance) != null;
  }

  getPointRating(exerciseResult: ExerciseProgress, taskInstance: TaskInstance) {
    return exerciseResult.submission?.pointRatings.find(r => r.taskInstance.task.id == taskInstance.task.id);
  }

  applyFilter(val: boolean) {
    this.hideInvisible = val;
    this.filteredExerciseProgresses = this.hideInvisible 
      ? this.exerciseProgresses.filter(r => !this.hideInvisible ||  r.exercise.visible)
      : this.exerciseProgresses;
  }
}
