import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { SearchResult } from '../../models/search-result';
import { SearchService } from '../../services/search.service';
import { combineLatest } from 'rxjs';
import { takeUntil, flatMap, filter } from 'rxjs/operators';
import { AuthService } from 'app/core/services/auth.service';
import { AppStateService } from 'app/core/services/app.state.service';

@Component({
  selector: 'ngx-search-results',
  templateUrl: './search-results.component.html',
  styleUrls: ['./search-results.component.scss']
})
export class SearchResultsComponent extends BaseComponent implements OnInit {
  loading: boolean = true;
  searchResult: SearchResult;

  constructor(
    private authService: AuthService,
    private appStateService: AppStateService,
    private searchService: SearchService,
    private route: ActivatedRoute
  ) { 
    super();
  }

  ngOnInit(): void {
    combineLatest(this.route.queryParamMap, this.authService.principal$, this.appStateService.term$)
    .pipe(
      takeUntil(this.unsubscribe),
      flatMap(([params, principal, term]) => {
        let query = params.get("q");
        return this.searchService.search(query, null);
      })
    )
    .subscribe(searchResult => {
      this.searchResult = searchResult;
      this.loading = false;
    })
  }
}
