import { User } from "app/core/models/user";
import { FileTree } from "app/shared/models/file-tree";

import * as moment from 'moment';
import { EvalResult } from "./eval-result";
import { EvalRating } from "./eval-rating";
import { SolutionComment } from "./solution-comment";
import { Iso8601Helper } from "app/shared/helper/iso8601.helper";
import { Task } from "app/features/tasks/models/task";
import { Exercise } from "app/features/exercises/models/exercise";
import { Submission } from "app/features/exercises/models/submission";

export class Solution {
    id: number;
    exercise: Exercise;
    task: Task;
    submittedBy: User | null;
    tree: FileTree;
    submittedAt: moment.Moment;
    evalResult: EvalResult;
    submission: Submission;
    delayed: boolean;
    attemptsExceeded: boolean;
    comments: SolutionComment[];
    commentsCount: number;

    constructor(solution: any) {
        this.id = solution.id;
        this.exercise = solution.exercise ? new Exercise(solution.exercise) : null;
        this.task = solution.task ? new Task(solution.task) : null;
        this.submittedBy = solution.submittedBy ? new User(solution.submittedBy) : null;
        this.tree = solution.tree ? new FileTree(solution.tree) : null;
        this.submittedAt = Iso8601Helper.fromString(solution.submittedAt);
        this.evalResult = solution.evalResult ? new EvalResult(solution.evalResult) : null;
        this.submission = solution.submission ? new Submission(solution.submission) : null;
        this.delayed = solution.delayed;
        this.attemptsExceeded = solution.attemptsExceeded;
        this.commentsCount = solution.commentsCount;
        this.comments = solution.comments ? solution.comments.map(c => new SolutionComment(c)) : [];
    }

    get hasResult() {
        return this.evalResult != null;
    }

    get isGood() {
        return this.hasResult && this.evalResult.rating == EvalRating.Good;
    }

    get isOk() {
        return this.hasResult && this.evalResult.rating == EvalRating.Ok;
    }

    get isBad() {
        return this.hasResult && this.evalResult.rating == EvalRating.Bad;
    }
}