import { Breadcrumb } from "app/shared/components/breadcrumbs/breadcrumbs.component";
import { Solution } from "../../models/solution";

export class SolutionBreadcrumbBuilder {
    static getForSolution(solution: Solution): Breadcrumb[] {
        if (solution.exercise) {
            return this.forExercise(solution);
        } else {
            return this.forTask(solution);
        }
    }

    static forTask(solution: Solution) {
        return [
            {
                id: 'pools',
            },
            {
                id: 'pool',
                params: {
                    pool_id: solution.task.pool.id
                },
                ctx: solution.task.pool
            },
            {
                id: 'task',
                params: {
                    task_id: solution.task.id
                },
                ctx: solution.task
            },
            {
                id: 'solutions',
                params: {
                    solution_id: solution.id
                },
                ctx: solution
            }
        ];
    }

    static forExercise(solution: Solution) {
        return [
            {
                id: 'exercises',
                params: {
                    course_id: solution.exercise.course.id
                },
            },
            {
                id: 'exercise',
                params: {
                    exercise_id: solution.exercise.id
                },
                ctx: solution.exercise
            },
            {
                id: 'submitSolution',
                params: {
                    exercise_id: solution.exercise.id,
                    task_id: solution.task.id
                },
                ctx: { exercise: solution.exercise, task: solution.task }
            },
            {
                id: 'solutions',
                params: {
                    solution_id: solution.id
                },
                ctx: solution
            }
        ];
    }
}