import { Component, OnInit, HostBinding, Input, EventEmitter, Output } from '@angular/core';
import { UntypedFormGroup, UntypedFormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { Task } from 'app/features/tasks/models/task';
import { FormComponent } from 'app/shared/components/base/form.component';
import { NotificationService } from 'app/shared/services/notification.service';
import { flatMap, takeUntil } from 'rxjs/operators';
import { TaskService } from 'app/features/tasks/services/task.service';
import { TaskPoolService } from '../../services/task-pool.service';
import {DropzoneComponent} from 'app/shared/components/dropzone/dropzone.component'
import { BackendError } from 'app/shared/models/backend-error';
import { TaskPool } from '../../models/task-pool';
import { environment } from 'environments/environment';

@Component({
  selector: 'ngx-task-upload',
  templateUrl: './task-upload.component.html',
  styleUrls: ['./task-upload.component.scss']
})
export class TaskUploadComponent extends FormComponent implements OnInit {
  @Input() pool: TaskPool;
  @Output() taskUploaded = new EventEmitter<Task>();

  importing: boolean;
  fileForm: UntypedFormGroup;
  task: Task;
  file: File;

  stefDocsUrl = environment.stefDocsUrl;

  constructor(
    private taskPoolService: TaskPoolService,
    private router: Router,
    private route: ActivatedRoute,
    private notificationService: NotificationService,
    private fb: UntypedFormBuilder) {
    super();
  }

  ngOnInit(): void {
    this.createForm();
  }

  createForm() {
    this.fileForm = this.fb.group({
      file: ['', Validators.required],
    });
  }

  handleFile(file: File) {
    this.file = file;
    this.fileForm.patchValue({
      file: file
    });
  }
  
  save() {
    this.importing = true;

    this.taskPoolService.importTask(this.pool.id, this.fileForm.get('file').value)
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(task => {
        this.notificationService.showSuccess("Aufgabe erfolgreich importiert");
        this.taskUploaded.emit(task);
        this.importing = false;
        this.file = null;
      }, (error: any) => {
        this.importing = false;

        if (BackendError.isBackendError(error)) {
          this.notificationService.showError(
            error.format(
              "Fehler beim Importieren der Aufgabe" 
            )
          );
        } else {
          throw error;
        }
      });
  }
}
