import { Pipe, PipeTransform } from "@angular/core";
import { BaseComponent } from "app/shared/components/base/base.component";
import { DownloadService } from "app/shared/services/download.service";
import { environment } from "environments/environment";
import { Observable, forkJoin, of } from "rxjs";
import { takeUntil, map } from "rxjs/operators";
import { Task } from "../models/task";

@Pipe({
    name: 'loadMedia'
})
export class LoadMediaPipe extends BaseComponent implements PipeTransform {
    constructor(private downloadService: DownloadService) {
        super();
    }

    transform(html: string): Observable<string> {
        let parser = new DOMParser();
        let doc = parser.parseFromString(html, 'text/html');
        var images = doc.querySelectorAll("img");

        let observables: Observable<void>[] = [];

        images.forEach(img => {
            let src = img.attributes.getNamedItem('src').value;
            if (src.startsWith(environment.backendUrl)) {
                observables.push(this.downloadService.downloadFileAsUrl(src, true)
                    .pipe(
                        map(([url, fileName]) => {
                            img.attributes.getNamedItem('src').value = url;
                        })
                    ));
            }
        })

        if (observables.length != 0) {
            return forkJoin(observables)
                .pipe(
                    takeUntil(this.unsubscribe),
                    map(() => doc.body.innerHTML)
                );
        } else {
            return of(html);
        }
    }
}