import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { FileTree } from 'app/shared/models/file-tree';

interface TreeNode<T> {
  data: T;
  children?: TreeNode<T>[];
  expanded?: boolean;
}

@Component({
  selector: 'ngx-file-browser',
  templateUrl: './file-browser.component.html',
  styleUrls: ['./file-browser.component.scss']
})
export class FileBrowserComponent implements OnInit {
  @Output() treeSelectedEvent = new EventEmitter<FileTree>();
  @Input() tree: FileTree;

  data: TreeNode<FileTree>[] = [];

  customColumn = 'filename';
  defaultColumns = [];
  allColumns = [ this.customColumn, ...this.defaultColumns ];
  
  constructor() { }

  private treeNodeFor(tree: FileTree): TreeNode<FileTree> {
    return {
      data: tree,
      children: tree.children.map(child => this.treeNodeFor(child)),
      expanded: false
    };
  }

  ngOnInit(): void {
    this.data = this.tree.children.map(tree => this.treeNodeFor(tree));
  }

  onTreeClicked(tree: FileTree) {
    if(tree.children.length === 0) {
      this.treeSelectedEvent.emit(tree);
    }
  }
}
