package de.hsrm.sls.subato.aor.term.imp.unit;

import de.hsrm.sls.subato.aor.term.model.Lecturer;
import de.hsrm.sls.subato.aor.term.model.ModulePartInstance;
import de.hsrm.sls.subato.courses.unit.CourseUnit;
import de.hsrm.sls.subato.courses.unit.CourseUnitRepository;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.UserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

@Service
@Transactional
public class LecturerImportService {
    @Autowired
    UserRepository userRepository;

    @Autowired
    CourseUnitRepository courseUnitRepository;

    public List<User> importLecturers(ModulePartInstance modulePartInstance) {
        var users = new ArrayList<User>();
        for(var lecturer : modulePartInstance.getLecturers()) {
            var user = getOrCreateLecturer(lecturer);
            users.add(user);
        }

        return users;
    }

    private User getOrCreateLecturer(Lecturer lecturer) {
        if (lecturer.getUser().getHdsName() != null) {
            var hdsLoginName = lecturer.getUser().getHdsName();
            var user = userRepository.findByLoginNameIgnoreCase(hdsLoginName).orElse(null);
            if (user == null) {
                user = new User();
                user.setLoginName(hdsLoginName);
                user.setFirstName(lecturer.getFirstName());
                user.setLastName(lecturer.getLastName());
                user.setMail(lecturer.getEmail());
            }

            return userRepository.save(user);
        } else {
            var user = userRepository.findByMailIgnoreCase(lecturer.getEmail()).orElse(null);
            if (user == null) {
                user = new User();
                user.setFirstName(lecturer.getFirstName());
                user.setLastName(lecturer.getLastName());
                user.setMail(lecturer.getEmail());
            }

            return userRepository.save(user);
        }
    }

    private void setLecturers(CourseUnit courseUnit, ArrayList<User> users) {
        courseUnit.getLecturers().addAll(users);
        courseUnitRepository.save(courseUnit);
    }
}
