package de.hsrm.sls.subato.auth.policy.users;

import de.hsrm.sls.subato.auth.Role;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyRule;
import de.hsrm.sls.subato.courses.course.Course_;
import de.hsrm.sls.subato.courses.membership.CourseStaffMembership_;
import de.hsrm.sls.subato.exercises.exercise.Exercise_;
import de.hsrm.sls.subato.solutions.solution.Solution;
import de.hsrm.sls.subato.solutions.solution.Solution_;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.User_;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Component;

import javax.persistence.criteria.JoinType;

/**
 * The current user is allowed to view a certain user
 */
@Component
public class UserViewPolicy implements PolicyRule {
    @Override
    public boolean check(CheckPolicyRequest request) {
        if(!request.matches(UsersAction.USER_VIEW)) {
            return false;
        }

        var user = (User) request.getObj();
        return request.authenticated() && (user.getId() == request.getSubject().getId() || request.getSubject().hasRole(Role.Staff) || request.getSubject().hasAnyGroup());
    }

    @Override
    public Specification checkAsSpec(CheckPolicyRequest request) {
        if(!request.matches(UsersAction.USER_VIEW)) {
            return Specification.where(null);
        }

        return accessesOwnUser(request.getSubject()).or(isStaff(request.getSubject())).or(isCourseStaff(request.getSubject()));
    }

    public static Specification<User> accessesOwnUser(SubatoPrincipal principal) {
        return (ex, cq, cb) -> cb.equal(ex.get(User_.id), principal != null ? principal.getId() : -1);
    }

    public static Specification<User> isStaff(SubatoPrincipal principal) {
        return (ex, cq, cb) -> cb.equal(cb.literal(true), principal != null && principal.hasRole(Role.Staff));
    }

    public static Specification<User> isCourseStaff(SubatoPrincipal principal) {
        return (ex, cq, cb) -> cb.equal(cb.literal(true), principal != null && principal.hasAnyGroup());
    }
}
