package de.hsrm.sls.subato.courses.course;

import de.hsrm.sls.subato.courses.membership.CourseStaffMembership;
import de.hsrm.sls.subato.exercises.exercise.Exercise;
import de.hsrm.sls.subato.exercises.exercise.ExerciseFilter;
import de.hsrm.sls.subato.exercises.exercise.ExerciseRepository;
import de.hsrm.sls.subato.exercises.exercise.ExerciseService;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstanceFilter;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstanceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class CourseCopyService {
    @Autowired
    ExerciseService exerciseService;

    @Autowired
    TaskInstanceService taskInstanceService;

    @Autowired
    ExerciseRepository exerciseRepository;

    public void copyCourse(Course source, Course target) {
        target.setDescription(source.getDescription());
        target.setSubmitPolicy(source.getSubmitPolicy());
        target.setShowEmails(source.isShowEmails());

        for(var m : source.getMemberships()) {
            var membership = new CourseStaffMembership(m.getUser(), target, m.getGroup());
            if(!target.getMemberships().contains(membership)) {
                target.getMemberships().add(membership);
            }
        }

        copyExercises(source, target);
    }

    private void copyExercises(Course source, Course target) {
        var exercises = exerciseService.findAll(new ExerciseFilter().courseId(source.getId()));
        for(var ex : exercises) {
            var exercise = new Exercise();
            exercise.setSummary(ex.getSummary());
            exercise.setName(ex.getName());
            exercise.setCourse(target);
            exercise.setUsingPoints(ex.isUsingPoints());
            exercise.setNumberC(ex.getNumberC());
            exercise.setDescription(ex.getDescription());

            var taskInstances = taskInstanceService.findAll(new TaskInstanceFilter().exerciseId(ex.getId()));
            for(var ti : taskInstances) {
                var taskInstance = new TaskInstance();
                taskInstance.setExercise(exercise);
                taskInstance.setTask(ti.getTask());
                taskInstance.setOrderNumber(ti.getOrderNumber());
                taskInstance.setReachablePoints(ti.getReachablePoints());
                exercise.getTaskInstances().add(taskInstance);
            }

            exercise = exerciseRepository.save(exercise);
            target.getExercises().add(exercise);
        }
    }
}
