package de.hsrm.sls.subato.seed;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.PropertyNamingStrategy;
import de.hsrm.sls.subato.seed.course.CourseSeedService;
import de.hsrm.sls.subato.seed.exercise.ExerciseSeedService;
import de.hsrm.sls.subato.seed.language.LanguageSeedService;
import de.hsrm.sls.subato.seed.taskpool.TaskPoolSeedService;
import de.hsrm.sls.subato.seed.term.TermSeedService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.nio.file.Path;

@Service
@Transactional
public class SeedService {
    @Autowired
    private LanguageSeedService languageSeedService;

    @Autowired
    private TaskPoolSeedService taskPoolSeedService;

    @Autowired
    private TermSeedService termSeedService;

    @Autowired
    private ExerciseSeedService exerciseSeedService;

    @Autowired
    private CourseSeedService courseSeedService;

    @Autowired
    private CoreDataSeedService coreDataSeedService;

    @Autowired
    private TermDataSeedService termDataSeedService;

    @Autowired
    private ObjectMapper objectMapper;

    public void seed(String seedDirPath) {
        var seedDir = Path.of(seedDirPath).toFile();
        if(!seedDir.exists()) {
            return;
        }

        var mapper =  objectMapper.copy();
        mapper.setPropertyNamingStrategy(PropertyNamingStrategy.SNAKE_CASE);

        termSeedService.seedTerms(seedDir, mapper);
        coreDataSeedService.seedCoreData(seedDir);
        courseSeedService.seedCourses(seedDir, mapper);
        termDataSeedService.seedTermData(seedDir);
        languageSeedService.seedLanguages(seedDir, mapper);
        taskPoolSeedService.seedTaskPools(seedDir, mapper);
        exerciseSeedService.seedExercises(seedDir, mapper);
    }
}
