package de.hsrm.sls.subato.seed.language;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.PropertyNamingStrategy;
import de.hsrm.sls.subato.seed.SeedingException;
import de.hsrm.sls.subato.tasks.lang.Language;
import de.hsrm.sls.subato.tasks.lang.LanguageRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.util.List;

@Service
@Transactional
public class LanguageSeedService {
    @Autowired
    LanguageRepository languageRepository;

    public void seedLanguages(File seedDir, ObjectMapper objectMapper) {
        try {
            var termsPath = seedDir.toPath().resolve("languages.json");
            var json = Files.readString(termsPath, StandardCharsets.UTF_8);
            var languageSeeds = objectMapper.readValue(json,  new TypeReference<List<LanguageSeed>>() {});

            for(var languageSeed : languageSeeds) {
                if(languageSeed.getId() != null && languageRepository.existsById(languageSeed.getId())) {
                    continue;
                }

                var language = new Language();
                language.setId(languageSeed.getId());
                language.setName(languageSeed.getName());

                languageRepository.save(language);
            }
        } catch (IOException e) {
            throw new SeedingException(e);
        }
    }
}
