package de.hsrm.sls.subato.seed.term;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.PropertyNamingStrategy;
import de.hsrm.sls.subato.modules.term.Term;
import de.hsrm.sls.subato.modules.term.TermRepository;
import de.hsrm.sls.subato.seed.SeedingException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.util.List;

@Service
@Transactional
public class TermSeedService {
    @Autowired
    TermRepository termRepository;

    public void seedTerms(File seedDir, ObjectMapper objectMapper) {
        try {
            var termsPath = seedDir.toPath().resolve("terms.json");
            var json = Files.readString(termsPath, StandardCharsets.UTF_8);
            var termSeeds = objectMapper.readValue(json,  new TypeReference<List<TermSeed>>() {});

            for(var termSeed : termSeeds) {
                if(termSeed.getId() != null && termRepository.existsBySourceId(termSeed.getId())) {
                    continue;
                }

                var term = new Term();
                term.setSourceId(termSeed.getId());
                term.setName(termSeed.getName());
                term.setLecturesStartAt(termSeed.getLecturesStartAt());
                term.setLecturesEndAt(termSeed.getLecturesEndAt());

                termRepository.save(term);
            }
        } catch (IOException e) {
            throw new SeedingException(e);
        }
    }
}
