package de.hsrm.sls.subato.shared.fs;

import com.google.common.base.Strings;
import de.hsrm.sls.subato.shared.fs.FileTree;
import de.hsrm.sls.subato.shared.helper.FileHelper;
import de.hsrm.sls.subato.shared.mime.MimeDetector;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.nio.file.Path;

@Service
public class FileTreeService {
    @Autowired
    MimeDetector mimeDetector;

    public FileTree createTreeFrom(File file, String basePath, boolean recursive, boolean includeRoot, String contentTypeOverride) {
        if(!file.exists()) {
            return null;
        }

        var contentType = !Strings.isNullOrEmpty(contentTypeOverride) ? contentTypeOverride : mimeDetector.getContentType(file);
        var isSourceFile = FileHelper.isSourceFile(contentType);

        var fileName = includeRoot ? file.getName() : "";
        var item = new FileTree(fileName, Path.of(basePath).resolve(fileName).toString(), contentType, isSourceFile);

        if(file.isDirectory()) {
            item.setDirectory(true);

            if(recursive) {
                var relativePath = Path.of(basePath).resolve(Path.of(fileName));

                for (var f : file.listFiles()) {
                    item.getChildren().add(createTreeFrom(f, relativePath.toString(), recursive, true));
                }
            }
        }

        return item;
    }

    public FileTree createTreeFrom(File file, String basePath, boolean recursive, boolean includeRoot) {
        return createTreeFrom(file, basePath, recursive, includeRoot, null);
    }
}
