package de.hsrm.sls.subato.shared.mime;

import de.hsrm.sls.subato.aor.core.CoreDataImportRunner;
import org.apache.tika.Tika;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.io.File;

/**
 * Responsible for mapping file names to mime types.
 *
 * Uses the Apache Tika library for reliable detection.
 * However, some files are detected as application/octet-stream by tika, so we need to map the exceptions here
 */
@Component
public class MimeDetector {
    /**
     * configurable mapping rules
     */
    private MimeProperties mimeProperties;

    Logger logger = LoggerFactory.getLogger(MimeDetector.class);

    public MimeDetector(MimeProperties mimeProperties) {
        this.mimeProperties = mimeProperties;

        logger.info("Configured mime mappings: %s".formatted(mimeProperties.getMime()));
    }

    /**
     * @return actual content type or application/octet-stream if it can't be guessed
     */
    public String getContentType(String fileName) {
        for(var mimeMapping : mimeProperties.getMime()) {
            if(fileName.endsWith(mimeMapping.getMap())) {
                return mimeMapping.getAs();
            }
        }

        String contentType = new Tika().detect(fileName);
        return contentType;
    }

    public String getContentType(File file) {
        return getContentType(file.getName());
    }
}
