package de.hsrm.sls.subato.solutions.evaluation.evaluator;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class EvaluatorService {
    @Autowired
    EvaluatorRegistryService evaluatorRegistryService;

    @Autowired
    EvaluatorRepository evaluatorRepository;

    private Evaluator get(String id, String version) {
        var registry = evaluatorRegistryService.getRegistry();
        var entry = registry.stream().filter(e -> e.getId().equals(id)).findFirst().orElseThrow(() -> new EvaluatorNotFoundException("No evaluator found with id %s".formatted(id)));

        EvaluatorEntry.Version evaluatorVersion;
        if(version != null) {
            evaluatorVersion = entry.getVersions().stream().filter(v -> v.getId().equals(version)).findFirst().orElseThrow(() -> new EvaluatorNotFoundException("Version %s not found for evaluator %s".formatted(version, id)));
        } else {
            evaluatorVersion = entry.getVersions().stream().findFirst().orElseThrow(() -> new EvaluatorNotFoundException("No versions for evaluator %s found".formatted(id)));
        }

        var evaluator = evaluatorRepository.findByIdIgnoreCaseAndVersionIgnoreCase(id, version);
        if(evaluator == null) {
            evaluator = new Evaluator();
            evaluator.setId(id);
            evaluator.setVersion(version);
        }

        evaluator.setCompile(evaluatorVersion.getCapability().isCompile());
        evaluator.setStyle(evaluatorVersion.getCapability().isStyle());
        evaluator.setTest(evaluatorVersion.getCapability().isTest());

        return evaluator;
    }

    public Evaluator resolveReference(String reference) {
        var splitted = reference.split("@");
        var id = splitted[0];
        var version = splitted.length > 1 ? splitted[1] : null;

        return get(id, version);
    }

    public Evaluator refresh(Evaluator evaluator) {
        return get(evaluator.getId(), evaluator.getVersion());
    }
}
