package de.hsrm.sls.subato.tasks.feedback;

import de.hsrm.sls.subato.tasks.task.Task;
import de.hsrm.sls.subato.user.User;

import javax.persistence.*;
import java.time.LocalDateTime;

/**
 * Represents a feedback entry for a specific task, created by a user.
 * Used for collecting feedback to a specific from students in order to
 *  1.) know the feeled complexity
 *  2.) know the feeled comprehensibility
 *  3.) get comments about improving the task text or to hint at mistakes (spelling, logic)
 *
 * Each user can give feedback to a task only once. Repeated feedback should change the previous feedback entry.
 * todo: unique constraint for (task, user)
 */
@Entity
@Table(name = "task_feedback", uniqueConstraints={
        @UniqueConstraint(columnNames = {"task_id", "from_id"})
})
public class TaskFeedback {
    /**
     * internal use only
     */
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @ManyToOne
    private Task task;

    @ManyToOne
    private User from;

    private String comment;

    /**
     * ranging from 1 (low) to 5 (high)
     */
    private int complexity;

    /**
     * ranging from 1 (poor) to 5 (good)
     */
    private int comprehensibility;

    @Column(columnDefinition = "TIMESTAMP")
    private LocalDateTime submittedAt;

    /**
     * if the feedback was checked by the lecturer.
     */
    private boolean read;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Task getTask() {
        return task;
    }

    public void setTask(Task task) {
        this.task = task;
    }

    public User getFrom() {
        return from;
    }

    public void setFrom(User from) {
        this.from = from;
    }

    public String getComment() {
        return comment;
    }

    public void setComment(String comment) {
        this.comment = comment;
    }

    public int getComplexity() {
        return complexity;
    }

    public void setComplexity(int complexity) {
        this.complexity = complexity;
    }

    public int getComprehensibility() {
        return comprehensibility;
    }

    public void setComprehensibility(int comprehensibility) {
        this.comprehensibility = comprehensibility;
    }

    public LocalDateTime getSubmittedAt() {
        return submittedAt;
    }

    public void setSubmittedAt(LocalDateTime submittedAt) {
        this.submittedAt = submittedAt;
    }

    public boolean isRead() {
        return read;
    }

    public void setRead(boolean read) {
        this.read = read;
    }
}
