package de.hsrm.sls.subato.tasks.task;

import de.hsrm.sls.subato.exercises.taskinstance.TaskInstanceDetailDto;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.MethodParameter;
import org.springframework.http.MediaType;
import org.springframework.http.converter.json.MappingJacksonValue;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.servlet.mvc.method.annotation.AbstractMappingJacksonResponseBodyAdvice;

import java.net.URI;
import java.net.URISyntaxException;
import java.nio.file.Path;
import java.util.regex.Pattern;

/**
 * Fix relative urls to assets in the html content of the task,
 * so that the url points to our endpoint, serving the asset
 * @see TaskController#downloadAsset
 */
@RestControllerAdvice
public class FixReferencesAdvice extends AbstractMappingJacksonResponseBodyAdvice {
    private static Logger logger = LoggerFactory.getLogger(FixReferencesAdvice.class);

    String getBaseUrl(ServerHttpRequest req) {
        return ""
                + req.getURI().getScheme() + "://"
                + req.getURI().getHost()
                + ":" + req.getURI().getPort();
    }

    private String rewriteUrls(String content, String path, ServerHttpRequest request) {
        var url = "%s/%s".formatted(getBaseUrl(request), path);

        var matcher = Pattern.compile("src=\"(\\S+)\"").matcher(content);
        return matcher.replaceAll(r -> {
            var matchedUrl = r.group(1);
            var correctedUrl = matchedUrl;

            try {
                var uri = new URI(matchedUrl);
                if(!uri.isAbsolute()) {
                    correctedUrl ="%s%s".formatted(url, matchedUrl);
                }
            } catch (URISyntaxException e) {
                // there is an incorrect url in the task text, thus this is no error
            }

            return "src=\"%s\"".formatted(correctedUrl);
        });
    }

    @Override
    protected void beforeBodyWriteInternal(MappingJacksonValue bodyContainer, MediaType contentType, MethodParameter returnType, ServerHttpRequest request, ServerHttpResponse response) {
        var obj = bodyContainer.getValue();

        if(obj != null && obj instanceof TaskDetailDto) {
            var task = (TaskDetailDto) obj;
            var taskAssetPath = Path.of(
                    "task",
                    String.valueOf(task.getId()),
                    "asset?path=").toString();
            task.setContent(rewriteUrls(task.getContent(), taskAssetPath, request));
        }

        if(obj != null && obj instanceof TaskInstanceDetailDto) {
            var taskInstance = (TaskInstanceDetailDto) obj;
            if(taskInstance.getTask() != null) {
                var taskAssetPath = Path.of(
                        "exercise",
                        String.valueOf(taskInstance.getExercise().getId()),
                        "task",
                        String.valueOf(taskInstance.getTask().getId()), "asset?path=").toString();
                taskInstance.getTask().setContent(rewriteUrls(taskInstance.getTask().getContent(), taskAssetPath, request));
            }
        }
    }
}