package de.hsrm.sls.subato.aor.term;

import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.courses.course.CourseRepository;
import de.hsrm.sls.subato.modules.term.TermRepository;
import de.hsrm.sls.subato.user.UserRepository;
import org.junit.Assert;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ResourceUtils;

import java.nio.file.Files;
import java.util.List;

/**
 * ad-hoc integration tests, should be refactored someday to test the individual import steps isolated
 */
@SpringBootTest
@DirtiesContext(classMode = DirtiesContext.ClassMode.BEFORE_EACH_TEST_METHOD)
@AutoConfigureTestDatabase(replace = AutoConfigureTestDatabase.Replace.ANY)
@Tag(Tags.INTEGRATION)
@Transactional
public class TermDataImportManagerTests {
    @Autowired
    private TermDataImportManager termDataImportManager;

    @Autowired
    private CourseRepository courseRepository;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private TermRepository termRepository;

    @Test
    public void shouldCreateTermIfNotExists() throws Exception {
        var sampleImportFile = ResourceUtils.getFile("classpath:testfiles/aor/exportSS22.json");
        termDataImportManager.importFromJson(Files.readString(sampleImportFile.toPath()));

        var importedTerm = termRepository.findAll().stream().filter(t -> t.getName().equals("WS2223")).findFirst().orElse(null);
        Assert.assertNotNull(importedTerm);
    }

    @Test
    public void shouldCreateParticipantsAndLecturersIfNotExists() throws Exception {
        var sampleImportFile = ResourceUtils.getFile("classpath:testfiles/aor/exportWS2223_modgroup.json");
        var loginNames = List.of("stud1", "stud2", "stud3", "stud4", "lect1", "lect2");
        for(var loginName : loginNames) {
            Assert.assertNull(loginName, userRepository.findByLoginNameIgnoreCase(loginName).orElse(null));
        }

        termDataImportManager.importFromJson(Files.readString(sampleImportFile.toPath()));

        for(var loginName : loginNames) {
            Assert.assertNotNull(loginName, userRepository.findByLoginNameIgnoreCase(loginName).orElse(null));
        }
    }

    @Test
    public void shouldImportSimpleCourse() throws Exception {
        var d  = courseRepository.findById(2L).get();
        var sampleImportFile = ResourceUtils.getFile("classpath:testfiles/aor/exportWS2223_simple.json");
        termDataImportManager.importFromJson(Files.readString(sampleImportFile.toPath()));

        var course = courseRepository.findById(2L).get();
        CourseImportAssert.assertSimpleCourse(course);

        // nothing should change now
        termDataImportManager.importFromJson(Files.readString(sampleImportFile.toPath()));
        course = courseRepository.findById(2L).get();
        CourseImportAssert.assertSimpleCourse(course);
    }

    @Test
    public void shouldImportModGroupCourse() throws Exception {
        var sampleImportFile = ResourceUtils.getFile("classpath:testfiles/aor/exportWS2223_modgroup.json");
        termDataImportManager.importFromJson(Files.readString(sampleImportFile.toPath()));

        var course = courseRepository.findById(2L).get();
        CourseImportAssert.assertModGroupCourse(course);

        // nothing should change now
        termDataImportManager.importFromJson(Files.readString(sampleImportFile.toPath()));
        course = courseRepository.findById(2L).get();
        CourseImportAssert.assertModGroupCourse(course);
    }

    @Test
    public void shouldImportModGroupPartialCourse() throws Exception {
        var sampleImportFile = ResourceUtils.getFile("classpath:testfiles/aor/exportWS2223_modgroup_partial.json");

        termDataImportManager.importFromJson(Files.readString(sampleImportFile.toPath()));
        var course = courseRepository.findById(2L).get();
        CourseImportAssert.assertModGroupPartialCourse(course);
    }
}
